package net.t2code.automatedMessages.config;

import lombok.Getter;
import net.t2code.automatedMessages.Util;
import net.t2code.automatedMessages.objects.Message;
import net.t2code.automatedMessages.system.Main;
import net.t2code.t2codelib.SPIGOT.api.messages.T2Creplace;
import net.t2code.t2codelib.SPIGOT.api.minecraftVersion.T2CmcVersion;
import org.bukkit.Sound;
import org.bukkit.configuration.file.YamlConfiguration;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Objects;

public class FileBuild {

    @Getter
    private static HashMap<String, Message> messageHashMap = new HashMap<>();

    public static void selectConfig() {

        File config = new File(Main.getPath(), "config.yml");
        YamlConfiguration yamlConfiguration = YamlConfiguration.loadConfiguration(config);
        messageHashMap.clear();
        for (Config value : Config.values()) {
            switch (value.configParam) {
                case STRING:
                    if (!yamlConfiguration.contains(value.path)) {
                        yamlConfiguration.set(value.path, value.valueString);
                    }
                    value.valueString = T2Creplace.replace(Util.getPrefix(), Objects.requireNonNull(yamlConfiguration.getString(value.path)));
                    break;
                case BOOLEAN:
                    if (!yamlConfiguration.contains(value.path)) {
                        yamlConfiguration.set(value.path, value.valueBoolean);
                    }
                    value.valueBoolean = yamlConfiguration.getBoolean(value.path);
                    break;

                case INTEGER:
                    if (!yamlConfiguration.contains(value.path)) {
                        yamlConfiguration.set(value.path, value.valueInteger);
                    }
                    value.valueInteger = yamlConfiguration.getInt(value.path);
                    break;

                case MESSAGE:
                    if (!yamlConfiguration.contains(value.msgPath)) {
                        setExample("exampleDiscord", true,
                                "[prefix] <aqua>If you need help, just contact us on our <color:#ffcb2e><hover:show_text:'<red>dc.t2code.net</red>'><click:open_url:'https://discord.t2code.net'>T2Code Support Discord server</click></hover></color>.</aqua>",
                                true, sound(), false, "t2c.automatedmessages.messages.examplediscord", 20,
                                false, "*", "*", "*", "*", "*", yamlConfiguration);
                        setExample("exampleTime", true,
                                "[prefix] <aqua>Es ist jetzt <yellow>[time]</yellow></aqua>",
                                true, sound(), false, "t2c.automatedmessages.messages.exampletime", 0,
                                true, "0/15/30/45", "*", "*", "*", "*", yamlConfiguration);
                    }
                    for (String key : yamlConfiguration.getConfigurationSection(value.msgPath).getKeys(false)) {
                        Boolean enable = yamlConfiguration.getBoolean(value.msgEnablePath.replace("KEY", key));
                        String message = yamlConfiguration.getString(value.msgMsgPath.replace("KEY", key));
                        Boolean soundEnable = yamlConfiguration.getBoolean(value.soundEnablePath.replace("KEY", key));

                        Sound sound;
                        try {
                            sound = Sound.valueOf(yamlConfiguration.getString(value.soundPath.replace("KEY", key)));
                        } catch (Exception e) {
                            sound = sound();
                        }


                        Integer interval = yamlConfiguration.getInt(value.msgIntervalPath.replace("KEY", key));

                        Boolean permissionNeededToSeeMessage = yamlConfiguration.getBoolean(value.permissionNeededToSeeMessagePath.replace("KEY", key));
                        String permission = yamlConfiguration.getString(value.permissionPath.replace("KEY", key));

                        Boolean exactTime = yamlConfiguration.getBoolean(value.exactTimePath.replace("KEY", key));
                        String timeMinute = yamlConfiguration.getString(value.timeMinutePath.replace("KEY", key));
                        String timeHour = yamlConfiguration.getString(value.timeHourPath.replace("KEY", key));
                        String timeDayOfMonth = yamlConfiguration.getString(value.timeDayOfMonthPath.replace("KEY", key));
                        String timeMonth = yamlConfiguration.getString(value.timeMonthPath.replace("KEY", key));
                        String timeDayOfWeek = yamlConfiguration.getString(value.timeDayOfWeekPath.replace("KEY", key));
                        Message msg = new Message(key, enable, message, soundEnable, sound, permissionNeededToSeeMessage, permission, interval,
                                exactTime, timeMinute, timeHour, timeDayOfMonth, timeMonth, timeDayOfWeek);
                        messageHashMap.put(key, msg);
                    }
                    break;
            }
        }
        try {
            yamlConfiguration.save(config);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    static Sound sound() {
        if (T2CmcVersion.isMc1_8()) {
            return Sound.valueOf("NOTE_PIANO");
        } else if (T2CmcVersion.isMc1_9() || T2CmcVersion.isMc1_10() || T2CmcVersion.isMc1_11() || T2CmcVersion.isMc1_12()) {
            return Sound.valueOf("BLOCK_NOTE_HARP");
        } else return Sound.BLOCK_NOTE_BLOCK_HARP;
    }

    private static void setExample(String key, Boolean enable, String msg, Boolean soundEnable, Sound sound, Boolean permissionNeededToSeeMessage, String permission, Integer interval, Boolean exactTime, String timeMinute, String timeHour, String timeDayOfMonth,
                                   String timeMonth, String timeDayOfWeek, YamlConfiguration yamlConfiguration) {
        yamlConfiguration.set("messages." + key + ".enable", enable);
        yamlConfiguration.set("messages." + key + ".message", msg);
        yamlConfiguration.set("messages." + key + ".sound.enable", soundEnable);
        yamlConfiguration.set("messages." + key + ".sound.sound", sound.toString());
        yamlConfiguration.set("messages." + key + ".permissionNeededToSeeMessage.enable", permissionNeededToSeeMessage);
        yamlConfiguration.set("messages." + key + ".permissionNeededToSeeMessage.permission", permission);
        yamlConfiguration.set("messages." + key + ".intervalInMin", interval);
        yamlConfiguration.set("messages." + key + ".exactTime.enable", exactTime);
        yamlConfiguration.set("messages." + key + ".exactTime.timeMinute", timeMinute);
        yamlConfiguration.set("messages." + key + ".exactTime.timeHour", timeHour);
        yamlConfiguration.set("messages." + key + ".exactTime.timeDayOfMonth", timeDayOfMonth);
        yamlConfiguration.set("messages." + key + ".exactTime.timeMonth", timeMonth);
        yamlConfiguration.set("messages." + key + ".exactTime.timeDayOfWeek", timeDayOfWeek);
    }
}
